@@ -1,6 +1,6 @@
 """Allow users to use values directly."""
 from oblivious.oblivious import\
     native, sodium,\
-    scalar, rand, inv,\
-    pnt, base,\
-    mul, add, sub
+    point, scalar,\
+    scl, rnd, inv, smu,\
+    pnt, bas, mul, add, sub
