@@ -9,14 +9,20 @@ primitives.
 from importlib import import_module
 import base64
 from bitlist import bitlist
+import rbcl.bindings
 from fountains import fountains
 from unittest import TestCase # pylint: disable=C0411
 
-import oblivious.oblivious as oblivious
+from oblivious import oblivious
+
+# Constant for the number of input-output pairs to include in each test.
+TRIALS_PER_TEST = 16
 
 # To simulate an environment in which sodium is absent, some tests set
-# `oblivious.sodium` to `None`; the reference below is used for restoration.
-oblivious.sodium_restore = oblivious.sodium
+# `oblivious.sodium` to `None` or modify `oblivious.sodium._sodium`;
+# the references below are used for restoration.
+sodium_lib_restore = oblivious.sodium._lib # pylint: disable=W0212
+sodium_restore = oblivious.sodium
 
 def api_methods():
     """
@@ -64,8 +70,8 @@ def check_or_generate_operation(test, fun, lengths, bits): # pylint: disable=R17
     fs = fountains( # Generate the input bit stream.
         sum(lengths),
         seed=bytes(0), # This is also the default; explicit for clarity.
-        limit=256,
-        bits=bits, # Reference output bit vector.
+        limit=min(TRIALS_PER_TEST, (len(bits) * 4)),
+        bits=bits[:(TRIALS_PER_TEST // 4)], # Reference output bit vector.
         function=fun
     )
 
@@ -74,7 +80,24 @@ def check_or_generate_operation(test, fun, lengths, bits): # pylint: disable=R17
 
     test.assertTrue(all(fs)) # Check that all outputs match.
 
-def define_classes(cls, sodium_hidden=False): # pylint: disable=R0915
+def sodium_hidden_and_fallback(hidden=False, fallback=False):
+    """
+    Return binary wrapper class definition that conforms to the
+    scenario being tested.
+    """
+    # pylint: disable=W0212
+    if hidden:
+        oblivious.sodium = None
+    elif fallback:
+        oblivious.sodium = sodium_restore
+        oblivious.sodium._lib = rbcl.bindings
+        oblivious.sodium._call = oblivious.sodium._call_wrapped
+    else:
+        oblivious.sodium = sodium_restore
+        oblivious.sodium._lib = sodium_lib_restore
+        oblivious.sodium._call = oblivious.sodium._call_unwrapped
+
+def define_classes(cls, hidden=False, fallback=False): # pylint: disable=R0915
     """
     Define and return four classes of unit tests given a wrapper
     class (`native` or `sodium`) for primitive operations.
@@ -86,14 +109,14 @@ def define_classes(cls, sodium_hidden=False): # pylint: disable=R0915
         """
 
         def test_rnd(self):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
-            for _ in range(256):
+            sodium_hidden_and_fallback(hidden, fallback)
+            for _ in range(TRIALS_PER_TEST):
                 s = cls.rnd()
                 self.assertTrue(len(s) == 32 and cls.scl(s))
 
         def test_scl_none(self):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
-            for _ in range(256):
+            sodium_hidden_and_fallback(hidden, fallback)
+            for _ in range(TRIALS_PER_TEST):
                 s = cls.scl()
                 self.assertTrue(len(s) == 32 and cls.scl(s))
 
@@ -101,7 +124,7 @@ def define_classes(cls, sodium_hidden=False): # pylint: disable=R0915
                 self,
                 bits='4df8fe738c097afa7f255b10c3ab118eeb73e38935605042ccb7581c73f1e5e9'
             ):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
+            sodium_hidden_and_fallback(hidden, fallback)
             fun = lambda bs: bitlist([1 if cls.scl(bs) is not None else 0])
             return check_or_generate_operation(self, fun, [32], bits)
 
@@ -109,7 +132,7 @@ def define_classes(cls, sodium_hidden=False): # pylint: disable=R0915
                 self,
                 bits='41c07230000960b274044a0080a8018aa0114380150000028c2700006081e1e1'
             ):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
+            sodium_hidden_and_fallback(hidden, fallback)
             def fun(bs):
                 s = cls.scl(bs)
                 return cls.inv(s) if s is not None else bytes([0])
@@ -119,7 +142,7 @@ def define_classes(cls, sodium_hidden=False): # pylint: disable=R0915
                 self,
                 bits='2ca120487000010295804000850254008018000000008000080100008400000c'
             ):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
+            sodium_hidden_and_fallback(hidden, fallback)
             def fun(bs):
                 (s1, s2) = (cls.scl(bs[:32]), cls.scl(bs[32:]))
                 return\
@@ -129,22 +152,22 @@ def define_classes(cls, sodium_hidden=False): # pylint: disable=R0915
             return check_or_generate_operation(self, fun, [32, 32], bits)
 
         def test_pnt_none(self):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
-            for _ in range(256):
+            sodium_hidden_and_fallback(hidden, fallback)
+            for _ in range(TRIALS_PER_TEST):
                 self.assertTrue(len(cls.pnt()) == 32)
 
         def test_pnt(
                 self,
                 bits='baf12de24e54deae0aa116816bf5eee23b1168c78e892372e08a9884de9d4c1b'
             ):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
+            sodium_hidden_and_fallback(hidden, fallback)
             return check_or_generate_operation(self, cls.pnt, [64], bits)
 
         def test_bas(
                 self,
                 bits='00386671840148d05620421002a2110aa800e289010040404cb2101c20e165a0'
             ):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
+            sodium_hidden_and_fallback(hidden, fallback)
             def fun(bs):
                 s = cls.scl(bs)
                 return cls.bas(s) if s is not None else bytes([0])
@@ -154,7 +177,7 @@ def define_classes(cls, sodium_hidden=False): # pylint: disable=R0915
                 self,
                 bits='2c040004500080008180400080000008a1180020001080080211004000080040'
             ):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
+            sodium_hidden_and_fallback(hidden, fallback)
             def fun(bs):
                 (s1, s2) = (cls.scl(bs[:32]), cls.scl(bs[32:]))
                 return\
@@ -167,7 +190,7 @@ def define_classes(cls, sodium_hidden=False): # pylint: disable=R0915
                 self,
                 bits='28400040500000008480000020024c00211800080000800002110040ac001044'
             ):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
+            sodium_hidden_and_fallback(hidden, fallback)
             def fun(bs):
                 (s1, s2) = (cls.scl(bs[:32]), cls.scl(bs[32:]))
                 return\
@@ -180,7 +203,7 @@ def define_classes(cls, sodium_hidden=False): # pylint: disable=R0915
                 self,
                 bits='24210008500080028000000025020c08000001200000800002008002ac081040'
             ):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
+            sodium_hidden_and_fallback(hidden, fallback)
             def fun(bs):
                 (s1, s2) = (cls.scl(bs[:32]), cls.scl(bs[32:]))
                 return\
@@ -195,52 +218,52 @@ def define_classes(cls, sodium_hidden=False): # pylint: disable=R0915
         """
 
         def test_point_random(self):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
-            for _ in range(256):
+            sodium_hidden_and_fallback(hidden, fallback)
+            for _ in range(TRIALS_PER_TEST):
                 self.assertTrue(len(cls.point.random()) == 32)
 
         def test_point_bytes(
                 self,
                 bits='baf12de24e54deae0aa116816bf5eee23b1168c78e892372e08a9884de9d4c1b'
             ):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
+            sodium_hidden_and_fallback(hidden, fallback)
             return check_or_generate_operation(self, cls.point.bytes, [64], bits)
 
         def test_point_hash(
                 self,
                 bits='10cb044c737b034d5755f8ba0e29432745ed4fb1a78ea22a15b2d1113492841b'
             ):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
+            sodium_hidden_and_fallback(hidden, fallback)
             return check_or_generate_operation(self, cls.point.hash, [64], bits)
 
         def test_point_base(
                 self,
                 bits='00386671840148d05620421002a2110aa800e289010040404cb2101c20e165a0'
             ):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
+            sodium_hidden_and_fallback(hidden, fallback)
             def fun(bs):
                 s = cls.scalar.bytes(bs)
                 return cls.point.base(s) if s is not None else bytes([0])
             return check_or_generate_operation(self, fun, [32], bits)
 
         def test_point_base64(self):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
-            for _ in range(256):
+            sodium_hidden_and_fallback(hidden, fallback)
+            for _ in range(TRIALS_PER_TEST):
                 p = cls.point()
                 p_b64 = base64.standard_b64encode(p).decode('utf-8')
                 self.assertEqual(p.to_base64(), p_b64)
                 self.assertEqual(cls.point.from_base64(p_b64), p)
 
         def test_point(self):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
-            for _ in range(256):
+            sodium_hidden_and_fallback(hidden, fallback)
+            for _ in range(TRIALS_PER_TEST):
                 self.assertTrue(len(cls.point()) == 32)
 
         def test_point_rmul(
                 self,
                 bits='2c040004500080008180400080000008a1180020001080080211004000080040'
             ):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
+            sodium_hidden_and_fallback(hidden, fallback)
             def fun(bs):
                 (s1, s2) = (cls.scalar.bytes(bs[:32]), cls.scalar.bytes(bs[32:]))
                 return\
@@ -253,7 +276,7 @@ def define_classes(cls, sodium_hidden=False): # pylint: disable=R0915
                 self,
                 bits='2c040004500080008180400080000008a1180020001080080211004000080040'
             ):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
+            sodium_hidden_and_fallback(hidden, fallback)
             def fun(bs):
                 (s1, s2) = (cls.scalar.bytes(bs[:32]), cls.scalar.bytes(bs[32:]))
                 # Below, `*` invokes `scalar.__mul__`, which delegates to `mul`
@@ -268,7 +291,7 @@ def define_classes(cls, sodium_hidden=False): # pylint: disable=R0915
                 self,
                 bits='28400040500000008480000020024c00211800080000800002110040ac001044'
             ):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
+            sodium_hidden_and_fallback(hidden, fallback)
             def fun(bs):
                 (s1, s2) = (cls.scalar.bytes(bs[:32]), cls.scalar.bytes(bs[32:]))
                 return\
@@ -281,7 +304,7 @@ def define_classes(cls, sodium_hidden=False): # pylint: disable=R0915
                 self,
                 bits='24210008500080028000000025020c08000001200000800002008002ac081040'
             ):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
+            sodium_hidden_and_fallback(hidden, fallback)
             def fun(bs):
                 (s1, s2) = (cls.scalar.bytes(bs[:32]), cls.scalar.bytes(bs[32:]))
                 return\
@@ -291,8 +314,8 @@ def define_classes(cls, sodium_hidden=False): # pylint: disable=R0915
             return check_or_generate_operation(self, fun, [32, 32], bits)
 
         def test_scalar_random(self):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
-            for _ in range(256):
+            sodium_hidden_and_fallback(hidden, fallback)
+            for _ in range(TRIALS_PER_TEST):
                 s = cls.scalar.random()
                 self.assertTrue(len(s) == 32 and cls.scalar.bytes(s) is not None)
 
@@ -300,7 +323,7 @@ def define_classes(cls, sodium_hidden=False): # pylint: disable=R0915
                 self,
                 bits='4df8fe738c097afa7f255b10c3ab118eeb73e38935605042ccb7581c73f1e5e9'
             ):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
+            sodium_hidden_and_fallback(hidden, fallback)
             fun = lambda bs: bitlist([1 if cls.scalar.bytes(bs) is not None else 0])
             return check_or_generate_operation(self, fun, [32], bits)
 
@@ -308,20 +331,20 @@ def define_classes(cls, sodium_hidden=False): # pylint: disable=R0915
                 self,
                 bits='09991cc13ab3799d9c05e0c75968859298977fb7b78efa2dcb6e1689e927ac0e'
             ):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
+            sodium_hidden_and_fallback(hidden, fallback)
             return check_or_generate_operation(self, cls.scalar.hash, [32], bits)
 
         def test_scalar_base64(self):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
-            for _ in range(256):
+            sodium_hidden_and_fallback(hidden, fallback)
+            for _ in range(TRIALS_PER_TEST):
                 s = cls.scalar()
                 s_b64 = base64.standard_b64encode(s).decode('utf-8')
                 self.assertEqual(s.to_base64(), s_b64)
                 self.assertEqual(cls.scalar.from_base64(s_b64), s)
 
         def test_scalar(self):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
-            for _ in range(256):
+            sodium_hidden_and_fallback(hidden, fallback)
+            for _ in range(TRIALS_PER_TEST):
                 s = cls.scalar()
                 self.assertTrue(len(s) == 32 and cls.scalar.bytes(s) is not None)
 
@@ -329,7 +352,7 @@ def define_classes(cls, sodium_hidden=False): # pylint: disable=R0915
                 self,
                 bits='41c07230000960b274044a0080a8018aa0114380150000028c2700006081e1e1'
             ):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
+            sodium_hidden_and_fallback(hidden, fallback)
             def fun(bs):
                 s = cls.scalar.bytes(bs)
                 return s.inverse() if s is not None else bytes([0])
@@ -339,7 +362,7 @@ def define_classes(cls, sodium_hidden=False): # pylint: disable=R0915
                 self,
                 bits='41c07230000960b274044a0080a8018aa0114380150000028c2700006081e1e1'
             ):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
+            sodium_hidden_and_fallback(hidden, fallback)
             def fun(bs):
                 s = cls.scalar.bytes(bs)
                 return ~s if s is not None else bytes([0])
@@ -349,7 +372,7 @@ def define_classes(cls, sodium_hidden=False): # pylint: disable=R0915
                 self,
                 bits='2ca120487000010295804000850254008018000000008000080100008400000c'
             ):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
+            sodium_hidden_and_fallback(hidden, fallback)
             def fun(bs):
                 (s1, s2) = (cls.scalar.bytes(bs[:32]), cls.scalar.bytes(bs[32:]))
                 return\
@@ -364,74 +387,74 @@ def define_classes(cls, sodium_hidden=False): # pylint: disable=R0915
         """
 
         def test_types_point_random(self):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
+            sodium_hidden_and_fallback(hidden, fallback)
             p = cls.point.random()
             self.assertTrue(isinstance(p, cls.point))
 
         def test_types_point_bytes(self):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
+            sodium_hidden_and_fallback(hidden, fallback)
             (bs,) = fountains(64, limit=1)
             p = cls.point.bytes(bs)
             self.assertTrue(isinstance(p, cls.point))
 
         def test_types_point_hash(self):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
+            sodium_hidden_and_fallback(hidden, fallback)
             (bs,) = fountains(64, limit=1)
             p = cls.point.hash(bs)
             self.assertTrue(isinstance(p, cls.point))
 
         def test_types_point_base(self):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
+            sodium_hidden_and_fallback(hidden, fallback)
             p = cls.point.base(cls.scalar.random())
             self.assertTrue(isinstance(p, cls.point))
 
         def test_types_point_mul(self):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
+            sodium_hidden_and_fallback(hidden, fallback)
             (bs,) = fountains(32 + 64, limit=1)
             (s, p) = (cls.scalar.hash(bs[:32]), cls.point.hash(bs[64:]))
             self.assertTrue(isinstance(s * p, cls.point))
 
         def test_types_point_add(self):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
+            sodium_hidden_and_fallback(hidden, fallback)
             (bs,) = fountains(64 + 64, limit=1)
             (p0, p1) = (cls.point.hash(bs[:64]), cls.point.hash(bs[64:]))
             self.assertTrue(isinstance(p0 + p1, cls.point))
 
         def test_types_point_sub(self):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
+            sodium_hidden_and_fallback(hidden, fallback)
             (bs,) = fountains(64 + 64, limit=1)
             (p0, p1) = (cls.point.hash(bs[:64]), cls.point.hash(bs[64:]))
             self.assertTrue(isinstance(p0 - p1, cls.point))
 
         def test_types_scalar_random(self):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
+            sodium_hidden_and_fallback(hidden, fallback)
             self.assertTrue(isinstance(cls.scalar.random(), cls.scalar))
 
         def test_types_scalar_bytes(self):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
+            sodium_hidden_and_fallback(hidden, fallback)
             bs = bytes(cls.scalar.random())
             self.assertTrue(isinstance(cls.scalar.bytes(bs), cls.scalar))
 
         def test_types_scalar_hash(self):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
+            sodium_hidden_and_fallback(hidden, fallback)
             (bs,) = fountains(32, limit=1)
             self.assertTrue(isinstance(cls.scalar.hash(bs), cls.scalar))
 
         def test_types_scalar_invert(self):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
+            sodium_hidden_and_fallback(hidden, fallback)
             self.assertTrue(isinstance(~cls.scalar.random(), cls.scalar))
 
         def test_types_scalar_inverse(self):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
+            sodium_hidden_and_fallback(hidden, fallback)
             self.assertTrue(isinstance(cls.scalar.random().inverse(), cls.scalar))
 
         def test_types_scalar_mul_scalar(self):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
+            sodium_hidden_and_fallback(hidden, fallback)
             (s0, s1) = (cls.scalar.random(), cls.scalar.random())
             self.assertTrue(isinstance(s0 * s1, cls.scalar))
 
         def test_types_scalar_mul_point(self):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
+            sodium_hidden_and_fallback(hidden, fallback)
             (bs,) = fountains(32 + 64, limit=1)
             (s, p) = (cls.scalar.hash(bs[:32]), cls.point.hash(bs[64:]))
             self.assertTrue(isinstance(s * p, cls.point))
@@ -442,22 +465,22 @@ def define_classes(cls, sodium_hidden=False): # pylint: disable=R0915
         """
 
         def test_algebra_scalar_inverse_identity(self):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
-            for bs in fountains(32, limit=256):
+            sodium_hidden_and_fallback(hidden, fallback)
+            for bs in fountains(32, limit=TRIALS_PER_TEST):
                 s = cls.scl(bs)
                 if s is not None:
                     self.assertEqual(cls.inv(cls.inv(s)), s)
 
         def test_algebra_scalar_inverse_mul_cancel(self):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
-            for bs in fountains(32 + 64, limit=256):
+            sodium_hidden_and_fallback(hidden, fallback)
+            for bs in fountains(32 + 64, limit=TRIALS_PER_TEST):
                 (s0, p0) = (cls.scl(bs[:32]), cls.pnt(bs[32:]))
                 if s0 is not None:
                     self.assertEqual(cls.mul(cls.inv(s0), cls.mul(s0, p0)), p0)
 
         def test_algebra_scalar_mul_commute(self):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
-            for bs in fountains(32 + 32 + 64, limit=256):
+            sodium_hidden_and_fallback(hidden, fallback)
+            for bs in fountains(32 + 32 + 64, limit=TRIALS_PER_TEST):
                 (s0, s1, p0) = (cls.scl(bs[:32]), cls.scl(bs[32:64]), cls.pnt(bs[64:]))
                 if s0 is not None and s1 is not None:
                     self.assertEqual(
@@ -466,20 +489,20 @@ def define_classes(cls, sodium_hidden=False): # pylint: disable=R0915
                     )
 
         def test_algebra_point_add_commute(self):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
-            for bs in fountains(64 + 64, limit=256):
+            sodium_hidden_and_fallback(hidden, fallback)
+            for bs in fountains(64 + 64, limit=TRIALS_PER_TEST):
                 (p0, p1) = (cls.pnt(bs[:64]), cls.pnt(bs[64:]))
                 self.assertEqual(cls.add(p0, p1), cls.add(p1, p0))
 
         def test_algebra_point_add_sub_cancel(self):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
-            for bs in fountains(64 + 64, limit=256):
+            sodium_hidden_and_fallback(hidden, fallback)
+            for bs in fountains(64 + 64, limit=TRIALS_PER_TEST):
                 (p0, p1) = (cls.pnt(bs[:64]), cls.pnt(bs[64:]))
                 self.assertEqual(cls.add(cls.sub(p0, p1), p1), p0)
 
         def test_algebra_scalar_mul_point_mul_associate(self):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
-            for bs in fountains(32 + 32 + 64, limit=256):
+            sodium_hidden_and_fallback(hidden, fallback)
+            for bs in fountains(32 + 32 + 64, limit=TRIALS_PER_TEST):
                 (s0, s1, p0) = (cls.scl(bs[:32]), cls.scl(bs[32:64]), cls.pnt(bs[64:]))
                 if s0 is not None and s1 is not None:
                     self.assertEqual(
@@ -488,8 +511,8 @@ def define_classes(cls, sodium_hidden=False): # pylint: disable=R0915
                     )
 
         def test_algebra_scalar_mul_point_add_distribute(self):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
-            for bs in fountains(32 + 64 + 64, limit=256):
+            sodium_hidden_and_fallback(hidden, fallback)
+            for bs in fountains(32 + 64 + 64, limit=TRIALS_PER_TEST):
                 (s0, p0, p1) = (cls.scl(bs[:32]), cls.pnt(bs[32:96]), cls.pnt(bs[96:]))
                 if s0 is not None:
                     self.assertEqual(
@@ -498,12 +521,12 @@ def define_classes(cls, sodium_hidden=False): # pylint: disable=R0915
                     )
 
         def test_algebra_scalar_mul_scalar_on_right_hand_side_of_non_scalar(self):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
+            sodium_hidden_and_fallback(hidden, fallback)
             s = cls.scalar.random()
             self.assertRaises(TypeError, lambda: bytes([0]) * s)
 
         def test_algebra_scalar_mul_point_on_left_hand_side(self):
-            oblivious.sodium = None if sodium_hidden else oblivious.sodium_restore
+            sodium_hidden_and_fallback(hidden, fallback)
             s = cls.scalar.random()
             p = cls.point.hash(bytes([0]*32))
             self.assertRaises(TypeError, lambda: p * s)
@@ -517,9 +540,18 @@ def define_classes(cls, sodium_hidden=False): # pylint: disable=R0915
 
 # The instantiated test classes below are discovered by nose and
 # executed in alphabetical order.
-(Test_primitives_native_no_sodium, Test_classes_native_no_sodium,
- Test_types_native_no_sodium, Test_algebra_native_no_sodium) =\
-    define_classes(oblivious.native, sodium_hidden=True)
+(
+    Test_primitives_native_no_sodium,
+    Test_classes_native_no_sodium,
+    Test_types_native_no_sodium,
+    Test_algebra_native_no_sodium
+) = define_classes(oblivious.native, hidden=True)
+(
+    Test_primitives_sodium_rbcl_no_sodium,
+    Test_classes_sodium_rbcl_no_sodium,
+    Test_types_sodium_rbcl_no_sodium,
+    Test_algebra_sodium_rbcl_no_sodium
+) = define_classes(oblivious.sodium, fallback=True)
 (Test_primitives_native, Test_classes_native, Test_types_native, Test_algebra_native) =\
     define_classes(oblivious.native)
 (Test_primitives_sodium, Test_classes_sodium, Test_types_sodium, Test_algebra_sodium) =\
